#! /usr/bin/python

## <filename>
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt

## Parts of this module are based on pyhtk, developed at
## Canolfan Bedwyr and to which the following coyright notice applies:
##
## # Copyright (c) 2005, University of Wales, Bangor 
## # All rights reserved.
## # 
## # Redistribution and use in source and binary forms, with or without
## # modification, are permitted provided that the following conditions
## # are met:
## # 
## #   * Redistributions of source code must retain the above copyright
## # notice, this list of conditions and the following disclaimer.
## #   * Redistributions in binary form must reproduce the above copyright
## # notice, this list of conditions and the following disclaimer in the
## # documentation and/or other materials provided with the distribution.
## #   * Neither the name of the University of Wales, Bangor nor the names
## # of its contributors may be used to endorse or promote products
## # derived from this software without specific prior written permission.
## # 
## # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
## # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
## # A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
## # OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
## # SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
## # LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
## # DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
## # THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##
## For pyhtk see:
## http://www.bangor.ac.uk/~cbs007/pyhtk/README.html
## http://www.bangor.ac.uk/ar/cb/wispr.php
## http://www.e-gymraeg.org/wispr/

"""
========
trefnydd
========

Package:   trefnydd
Module:    lib.ambuilder
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

ambuilder
======

"""


import os
#from echo import echo

from trefnydd.utils import file_IO, location


class HMM(object):
    def __init__(self):
        pass

class Transp(object):
    def __init__(self, size):
        self.size = size
        self.matrix = [[0.5] * size] * size

    def __str__(self):
        out = ''
        for row in self.matrix:
            out = '%s%s\n' % (out, ' '.join([str(i) for i in row]))
        return out

class Transp5(Transp):
    def __init__(self):
        Transp.__init__(self, 5)
        self.matrix = [[0.0, 1.0, 0.0, 0.0, 0.0],
                       [0.0, 0.6, 0.4, 0.0, 0.0],
                       [0.0, 0.0, 0.6, 0.4, 0.0],
                       [0.0, 0.0, 0.0, 0.7, 0.3],
                       [0.0, 0.0, 0.0, 0.0, 0.0]]

    
class HTKHMM(HMM):
    def __init__(self, label=''):
        HMM.__init__(self)
        self.name = label
        self.states = [InitState(), HTKHMMState(), HTKHMMState(),
                       HTKHMMState(), FinState()]
        if len(self.states) == 5:
            self.transp = Transp5()
        else: self.transp = Transp(len(self.states))
        # transp: matrix of initial transition probabilities

    def __str__(self):
        out = '~h "%s"\n<BeginHMM>\n<NumStates> %d\n' \
              % (self.name, len(self.states))
        for i in range(1, len(self.states)-1):  #s in self.states[1:-1]:
            out = '%s<State> %d\n%s' % (out, i+1, self.states[i].__str__())
        out = '%s<TransP> %d\n%s' % (out, len(self.states),
                                     self.transp.__str__())
        out = '%s<EndHMM>\n' % out
        return out

class HMMState(object):
    def __init__(self):
        self.size = 39
        self.means = [0.0] * self.size
        self.variances = [1.0] * self.size

class HTKHMMState(HMMState):
    def __init__(self):
        HMMState.__init__(self)

    def __str__(self):
        
        out = '<Mean> %d\n%s\n<Variance> %d\n%s\n' \
              % (self.size, self.prettyPrint(self.means),
                 self.size, self.prettyPrint(self.variances))
        return out

    def prettyPrint(self, items):
        return ' '.join([str(i) for i in items])
        

class InitState(HTKHMMState):
    def __init__(self):
        HTKHMMState.__init__(self)

    def __str__(self): return ''

class FinState(HTKHMMState):
    def __init__(self):
        HTKHMMState.__init__(self)

    def __str__(self): return ''

class AcousticModel(location.Located):
    def __init__(self, debug=False):
        from trefnydd.lib.corpus import SpeechCorpus
        location.Located.__init__(self, debug)
        self.debug = debug
        self.name = ''
        self.corpus = SpeechCorpus()
        self.labelType = None
        self.hmms = set()

    def save(self):
        pass

    def load(modelHomeDir):
        pass

    def setup(self, corpus=None):
        pass

    def extractFeatures(self):
        pass

    def defineModelParameters(self):
        pass

    def trainModel(self):
        pass

class HTKAcousticModel(AcousticModel):
    def __init__(self, debug=False):
        AcousticModel.__init__(self, debug)
        self.name = 'htk'
        self.log = [] # list of rst snippets
        self._dirs = {
            'hmm0': 'hmm0',
            'lab': 'lab',
            'log': 'log',
            'mfc': 'mfc',
            'results': 'results',
            'wav': 'wav',
            'gram': 'gram',
            'wdnet': 'wdnet'
            }

        self._fns = {
            'config': 'config',
            'convert_scp': 'convert.scp',
            'hmmlist': 'HmmList',
            'labels_mlf': 'labels.mlf',
            'macros': 'macros',
            'trainlist': 'trainlist',
            'dict': 'dict',
            'hmmdefs': 'hmmdefs'
            }

        self.config = {'SOURCEKIND': 'WAVEFORM',
              'SOURCEFORMAT': 'WAV',
              'SOURCERATE': 625,
              'TARGETKIND': 'MFCC_0_D_A',
              'TARGETRATE': 100000,
              'SAVECOMPRESSED': 'T',
              'SAVEWITHCRC': 'T',
              'WINDOWSIZE': 250000,
              'USEHAMMING': 'T',
              'PREEMCOEF': 0.97,
              'NUMCHANS': 26,
              'CEPLIFTER': 22,
              'NUMCEPS': 12,
              'ENORMALISE': 'T'}

    def load(self, modelHomeDir):
        self.homeDir = modelHomeDir
        for root, dirs, fns in os.walk(self.homeDir):
            for dirn in dirs:
                self._dirs[dirn] = dirn
            for fn in fns:
                self._fns[fn] = fn
            break
            
    
    def setup(self, corp=None):
        os.chdir(self.homeDir)
        if corp: self.corpus = corp
        self.make_dirs()
        self.make_mlf()
        self.make_hmmlist()
        self.make_macros()

    def cleanup(self, dirList):
        for dirn in dirList:
            if dirn in self._dirs:
                for fn in os.listdir(self._dirs[dirn]):
                    fn = os.path.join(self.homeDir, self._dirs[dirn], fn)
                    #if self.debug: print fn
                    os.remove(fn)

    def make_dirs(self):
        for dirn in self._dirs:
            self.make_dir(dirn)
        for utt in self.corpus:
            os.symlink(utt.audioFn,
                       os.path.join(self._dirs['wav'],
                                    os.path.basename(utt.audioFn)))
            open(os.path.join(self._dirs['lab'],
                              '%s.lab' % utt.name),
                 'w').write(utt.write_HTKLab())

    def make_dir(self, dirn, name=''):
        if not name: name = self._dirs[dirn]
        self._dirs[dirn] = file_IO.getDir(os.path.join(self.homeDir,
                                                       name))

    def make_mlf(self):
        """Make Master Label File
        """
        out = u'#!MLF!#\n'
        for utt in self.corpus:
            lf = utt.write_HTKLab(self.labelType)
            out = u'%s%s' % (out, utt.write_HTKLab(self.labelType))
        self.export2file(out, 'labels_mlf')
    
    def make_hmmlist(self):
        self.corpus.makeIndex(self.labelType)
        labList = self.corpus.indices[self.labelType].keys()
        labList.sort()
        for lab in labList:
            self.hmms.add(HTKHMM(lab))
        out = '"%s"\n' % '"\n"'.join(labList)
        self.export2file(out, 'hmmlist')

    def make_convert_scp(self):
        """line format: wav/punk1.wav mfc/punk1.mfc"""
        mfcDir = self._dirs['mfc']
        out = ''
        for utt in self.corpus:
            wavfn = utt.audioFn
            mfcfn = os.path.join(mfcDir, ''.join([utt.name, '.mfc']))
            out = '%s%s   %s\n' % (out, wavfn, mfcfn)
        self.export2file(out, 'convert_scp')

    def make_trainlist(self):
        """line format:
        C:\Autosegmentation\HTK\HINIT\TrainingData\mfc\punk1.mfc"""
        mfcDir = self._dirs['mfc']
        out = ''
        for utt in self.corpus:
            mfcfn = os.path.join(mfcDir, ''.join([utt.name, '.mfc']))
            out = '%s%s\n' % (out, mfcfn)
        self.export2file(out, 'trainlist')

    def make_config(self):
        """\
        kHz     SOURCERATE
        =====   =====
        16000   625
        44100   226.757369614512
        """
        # todo/danger: assumes all wavs in corpus have same sample rate
        self.config['SOURCERATE'] = 10000000.0/self.corpus[0].audioSampleRate
        out = '\n'.join(['%s = %s' % (i[0], i[1])
                         for i in self.config.items()])
        out = '# Coding parameters\n%s\n' % out
        self.export2file(out, 'config')
    
    def make_macros(self):
        self.macros = HTKMacroSet()
        self.macros['o'] = """<STREAMINFO> 1 39
<VECSIZE> 39<NULLD><MFCC_0_D_A><DIAGC>"""
        
        self.macros['v']['varFloor1'] = """<Variance> 39
8.713753e-001 6.625333e-001 4.690533e-001 6.571800e-001 4.601170e-001 1.138707e+000 6.161002e-001
5.646990e-001 6.854044e-001 5.973042e-001 4.259520e-001 3.921174e-001 1.099038e+000 3.838177e-002
3.454069e-002 2.658178e-002 3.022736e-002 2.379769e-002 5.005486e-002 3.312984e-002 3.214758e-002
3.607899e-002 3.012902e-002 2.400114e-002 2.118599e-002 3.008951e-002 5.342365e-003 5.642878e-003
4.493490e-003 4.974959e-003 4.013374e-003 7.861852e-003 5.684706e-003 5.699892e-003 6.212835e-003
5.134702e-003 4.360192e-003 3.754829e-003 4.688293e-003
"""
        self.export2file(self.macros.__str__(), 'macros')

    def make_prototypes(self):
        for hmm in self.hmms:
            fullFn = os.path.join(self.homeDir,
                                  self._dirs['hmm0'],
                                  hmm.name)
            open(fullFn, 'w').write(hmm.__str__())
    
    def export2file(self, out, fn):
        fullFn = os.path.join(self.homeDir, self._fns[fn])
        open(fullFn, 'w').write(out.encode('utf-8'))
        
    def extractFeatures(self):
        """
        """
        self.log.append(self.makeHeading('Extract Features'))
        self.make_config()
        self.make_convert_scp()
        from trefnydd.lib.htk.commands import HCopy
        cmd = HCopy(self.debug)
        cmd.switches = {'-C': self._fns['config'],
                        '-S': self._fns['convert_scp'],
                        '-T': 1}
        cmd.run()
        self.log.append(cmd.log)

    def makeHeading(self, text, char='='):
        line = char * len(text)
        return '%s\n%s\n%s' % (line, text, line)

    def defineModelParameters(self):
        """
        """
        self.log.append(self.makeHeading('Define Model Parameters'))
        self.make_trainlist()
        self.make_prototypes()
        self.make_dir('hmm1', 'hmm1')
        from trefnydd.lib.htk.commands import HInit
        cmd = HInit(self.debug)
        for label in [h.name for h in self.hmms]:
            cmd.switches = {'-H': self._fns['macros'],
                            '-I': self._fns['labels_mlf'],
                            '-L': self._dirs['lab'],
                            '-M': self._dirs['hmm1'],
                            '-S': self._fns['trainlist'],
                            '-T': 1,
                            '-l': label}
            cmd.argument = '%s/%s' % (self._dirs['hmm0'], label)
            cmd.run()
            self.log.append(cmd.log)
        
    def trainModel(self, iters=4):
        """
        modelInDir = highest rated hmmN dir
        modelOutDir = new hmmN dir
        """
        from trefnydd.lib.htk.commands import HRest
        for i in range(iters):
            self.log.append(self.makeHeading('Train Model: iteration %d' % i))
            modelInDir, modelOutDir = self.getNextHMMDir()
            cmd = HRest(self.debug)
            for label in [h.name for h in self.hmms]:
                cmd.switches = {'-H': self._fns['macros'],
                                '-I': self._fns['labels_mlf'],
                                '-L': self._dirs['lab'],
                                '-M': self._dirs[modelOutDir],
                                '-S': self._fns['trainlist'],
                                '-l': label}
                cmd.argument = '%s/%s' % (self._dirs[modelInDir], label)
                cmd.run()
                self.log.append(cmd.log)

    def getNextHMMDir(self):
        hval = int(sorted([d for d in self._dirs
                           if d.startswith('hmm')])[-1][3:])
        a = 'hmm%d' % hval
        b = 'hmm%d' % (hval+1)
        self.make_dir(b, b)
        return a,b

    def pprintlog(self, format='rst'):
        format = format.lower()
        out = self.makeHeading('HTK Acoustic Model Builder Log', '~')
        out = u'%s\n\n.. contents::\n\n%s' % (out, '\n'.join(self.log))
        if format == 'html':
            from docutils.core import publish_string
            out = publish_string(out, writer_name='html',
                                 settings_overrides={'input_encoding': 'unicode',
                                                     'output_encoding': 'unicode'},
                                 )
        return out

    def getBestHMMdir(self):
        ### todo: refactor this hack
        hmmdirs = [d for d in self._dirs if d.startswith('hmm')]
        hmmdirs.sort()
        return os.path.join(self.homeDir, hmmdirs[-1])

    def get_hmmdefs(self):
        ### todo: refactor this hack
        hmmdefs = ''
        # get best hmm dir
        hd = self.getBestHMMdir()
        # put all hmms into string
        for fn in [f for f in os.listdir(hd)
                   if f != 'macros']:
            hmmdefs = '%s%s' % (hmmdefs, open(os.path.join(hd, fn)).read()) #\n
        # save string in file
        fn = os.path.join(self.homeDir, 'hmmdefs')
        open(fn, 'w').write(hmmdefs)
        # return path to file
        return fn

    def get_tiedlist(self):
        ### todo: refactor this hack
        # get best hmm dir
        hd = self.getBestHMMdir()
        # put all filenames (not macros, etc) in list
        tl = [f for f in os.listdir(hd) if f != 'macros']
        # save list in file
        fn = os.path.join(self.homeDir, 'tiedlist')
        open(fn, 'w').write('%s\n' % '\n'.join(tl))
        # return path to file
        return fn


class HTKMacroSet(dict):
    """** HTK Macros

See htkbook/node104_ct.html Macro Definitions for details.

Name   Description
====   ===========
~c     Shared choleski L' matrix
~d     Shared duration parameters
~h     Physical HMM
~i     Shared inverse full covariance matrix
~j     Linear transform
~l     Logical HMM
~m     Shared Gaussian mixture component
~o     Global option values
~p     Tied mixture
~r     Regression class tree
~s     Shared state distribution
~t     Shared transition matrix
~u     Shared mean vector
~v     Shared diagonal variance vector
~w     Shared stream weight vector
~x     Shared arbitrary transform matrix

    """
    _keys = ['o', 'c', 'd', 'h', 
             'i', 'j', 'l', 'm', 
             'p', 'r', 's', 't',
             'u', 'v', 'w', 'x']
    global_opts_key = _keys[0]

   
    def __init__(self):
        dict.__init__(self)
        for k in self._keys:
            self.setdefault(k, {})

    def __str__(self):
        out = '~%s\n%s\n\n' % (self.global_opts_key,
                               self[self.global_opts_key])
        keys = self._keys
        keys.remove(self.global_opts_key)
        for k in keys:
            if self[k]:
                for var in self[k].keys():
                    out = '%s~%s  %s\n%s\n\n' % (out, k, var, self[k][var])
        return out


class S3AcousticModel(AcousticModel):

    def __init__(self, debug=False):
        AcousticModel.__init__(self, debug)
        self.name = 's3'
        self.typeName = 's3_hmm'
        self.mdef_fn = ''
        self.means_fn = ''
        self.mixture_weigths_fn = ''
        self.variances_fn = ''
        self.transition_matrices_fn = ''

    def importFromHTKAM(self, model):
        # NB: requires htk2s3conv in path
        # todo: edit for reports into GUI
        #from htk2s3conv.htk_converter import *
        from trefnydd.extern.htk2s3conv.htk_converter import HtkConverter, HtkConverterError
        hmmdefs = model.get_hmmdefs()
        tiedlist = model.get_tiedlist()
        prefix = '%s.' % model.name
        prefixPath = os.path.join(self.name, prefix)
        currDir = os.getcwd()
        os.chdir(self.homeDir)
        os.chdir('..')
        try:
            if self.debug:
                print 's3am.importFromHTKAM::try block'
                print '\thmmdefs: %s\n\ttiedlist: %s\n\tprefix: %s' \
                      % (hmmdefs, tiedlist, prefixPath)
            conv = HtkConverter()
            conv.load(hmmdefs, tiedlist)
            #if self.debug: print 'models:\n\n' % conv.show()
            conv.writeS3(prefixPath)
        except HtkConverterError, e:
            print 'HtkConverterError: %s' %e
        os.chdir(currDir)
        self.mdef_fn = os.path.join(self.homeDir, '%smdef' % prefix)
        self.means_fn = os.path.join(self.homeDir, '%smean' % prefix)
        self.mixture_weights_fn = os.path.join(self.homeDir, '%smixw' % prefix)
        self.variances_fn = os.path.join(self.homeDir, '%svar' % prefix)
        self.transition_matrices_fn = os.path.join(self.homeDir, '%stmat' % prefix)


class S4AcousticModel(AcousticModel):
    #@echo
    def __init__(self, debug=False):
        AcousticModel.__init__(self, debug)
        self.name = 's4'
        self.modelProps = {
            'description': 'Acoustic models generated by Trefnydd',
            'modelClass': 'edu.cmu.sphinx.model.acoustic.NAME.Model',
            'modelLoader': 'edu.cmu.sphinx.model.acoustic.NAME.ModelLoader',
            'dataLocation': 'cd_continuous_8gau',
            'modelDefinition': 'etc/NAME.mdef',
            'isBinary': 'true',
            'featureType': '1s_c_d_dd',
            'vectorLength': 39,
            'sparseForm': 'false',
            'numberFftPoints': 512,
            'numberFilters': 40,
            'gaussians': 8,
            'minimumFrequency': 130,
            'maximumFrequency': 6800,
            'sampleRate': 16000}
        #if self.debug: print 'S4AM.__init__'

    #@echo
    def getS4src_dir(self):
        from trefnydd.meta.default import s4_src_dir
        return s4_src_dir

    s4src_dir = property(fget=getS4src_dir)

    #@echo
    def importFromS3AM(self, s3m):
        "see sphinx4/UsingSphinxTrainModels.html"
        self.name = '%s_s4' % s3m.name
        #if self.debug: print 'S4AM.importFromS3AM.name = %s' % self.name
        self.makePopulateModelDir(s3m) # test ok
        self.updateBuildXML()          # test ok
        self.run_ant()                 # test ok
        self.non_ant_deploy()          # test ok

    #@echo
    def makePopulateModelDir(self, s3model):
        """
        - mkdir:self.name
        - copy files from s3
        - create model.props
        """
        self.dictDir = file_IO.getDir(os.path.join(self.homeDir, 'dict'))
        self.etcDir = file_IO.getDir(os.path.join(self.homeDir, 'etc'))
        self.hmmDir = file_IO.getDir(os.path.join(self.homeDir,
                                                  s3model.typeName))
        self.libDir = file_IO.getDir(os.path.join(self.homeDir, 'lib'))
        self.importS3Files(s3model) # test ok
        self.makeModelProps()       # test ok
        #if self.debug: print 'S4AM.makePopulateModelDir.  Done.'

    #@echo
    def importS3Files(self, s3m): # test ok
        # todo: move copyFile to file_IO
        self.mdef_fn = self.copyFile(s3m.mdef_fn, self.etcDir)
        self.means_fn = self.copyFile(s3m.means_fn, self.hmmDir)
        self.mixture_weights_fn = self.copyFile(s3m.mixture_weights_fn,
                                                self.hmmDir)
        self.variances_fn = self.copyFile(s3m.variances_fn, self.hmmDir)
        self.transition_matrices_fn = self.copyFile(s3m.transition_matrices_fn,
                                                    self.hmmDir)

    #@echo
    def copyFile(self, srcFn, destDir): # test ok
        newFn = os.path.join(destDir, os.path.basename(srcFn))
        open(newFn, 'wb').write(open(srcFn).read())
        return newFn

    #@echo
    def makeModelProps(self): # test ok
        self.modelProps['modelDefinition'] = self.mdef_fn
        self.modelProps['modelClass'] = self.modelProps['modelClass'].replace('NAME', self.name)
        self.modelProps['modelLoader'] = self.modelProps['modelLoader'].replace('NAME', self.name)
        # write file
        out = 'description = %s\n\n' % self.modelProps['description']
        keys = self.modelProps.keys()
        keys.remove('description')
        keys.sort()
        for k in keys:
            out = '%s%s = %s\n' % (out, k, self.modelProps[k])
        open(os.path.join(self.homeDir, 'model.props'), 'w').write(out)
    
    #@echo
    def updateBuildXML(self): # test: not working
        # run ant manually: why isnt it finding jsapi!?
        """
        <!-- replace values with real names -->
        <property name="newAcousticModel_name"
                  value="TREFNYDD_GENERATED_AM_NAME"/>
        <!-- value = AM name -->
        
        <property name="newAcousticModel_data_dir"
                  value="TREFNYDD_GENERATED_AM_DATA_DIR"/>
        <!-- value = relative or absolute path to dir -->
        """
        from trefnydd.lib.s4.buildNewAMOnly import xml
        out = xml
        out = out.replace("TREFNYDD_S4_SRC_DIR", self.s4src_dir)
        out = out.replace("TREFNYDD_GENERATED_AM_NAME", self.name)
        out = out.replace("TREFNYDD_GENERATED_AM_DATA_DIR", self.homeDir)
        open(os.path.join(self.homeDir, 'buildNewAMOnly.xml'), 'w').write(out)

    #@echo
    def run_ant(self):
        """
        """
        currDir = os.getcwd()
        os.chdir(self.homeDir)
        ch_stdin, ch_stdout, ch_stderr = os.popen3('ant -f buildNewAMOnly.xml')
        stdout = ch_stdout.read()
        stderr = ch_stderr.read().strip()
        if stderr:
            self.print_error_report('Ant', stdout, stderr)
        os.chdir(currDir)

    def print_error_report(self, src_cmd, stdout, stderr):
        out = """* Error messages from %s\n\n
** stdout\n\n%s\n\n
** stderr\n\n%s\n\n""" \
        % (src_cmd, stdout, stderr)
        print out

    #@echo
    def non_ant_deploy(self):
        """Copy stuff from s4srcdir to self.homeDir
        from X: A.jar
        from Y: {Model,ModelLoader PropertiesDumper}.class
        """
        src_path = os.path.join(self.s4src_dir, 'lib', '%s.jar' % self.name)
        dst_path = os.path.join(self.homeDir, 'lib', '%s.jar' % self.name)
        os.rename(src_path, dst_path)

        local_path = 'bld/models/edu/cmu/sphinx/model/acoustic/%s' % self.name
        for jclass in ['Model', 'ModelLoader', 'PropertiesDumper']:
            classFn = '%s.class' % jclass
            src_path = os.path.join(self.s4src_dir, local_path, classFn)
            dst_path = os.path.join(self.homeDir,  classFn)
            os.rename(src_path, dst_path)


class CairoSpeechRecogResource(AcousticModel):
    def __init__(self, debug=False):
        AcousticModel.__init__(self, debug)

    def importFromS4AM(self, model):
        pass

    def save(self, saveDir):
        pass


def main():
    pass

if __name__ == '__main__':
    main()
 
