#! /usr/bin/python

## commands.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    lib.htk.commands
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

commands
========

class HTKCommand(object)  and subclasses

"""

import os

# from trefnydd.gui import gui

class HTKCommand(object):
    def __init__(self, debug=False):
        self.debug = debug
        self.name = ''
        self.usage = ''
        self.session = None # session with configs, fns, dirns, etc
        self._switches = {}
        self.switch_keys = []
        self.argument = ''
        # self.log = '' # ?
        self.cmdout = ''
        self.cmderr = ''

    def get_switches(self):
        return self._switches

    def set_switches(self, switches):
        for k in switches:
            if k in self.switch_keys:
                self._switches[k] = switches[k]
            else:
                raise StandardError, '%s: %s not a permissible option' \
                      % (self.name, k)

    switches = property(get_switches, set_switches)

    def get_cmd(self): #, switches=[]):
        switches = ' '.join(['%s %s' % (s[0], s[1])
                             for s in self.switches.items()])
        cmd = '%s %s' % (self.name, switches)
        if self.argument: cmd = '%s %s' % (cmd, self.argument)
        return cmd

    cmd = property(get_cmd)

    def run(self):
        stdin, stdout, stderr = os.popen3(self.cmd)
        self.cmdout = stdout.read()
        self.cmderr = stderr.read()
        if self.debug:
            out = self.getLog()
            if out: print out

    def getLog(self):
        line = '-' * len(self.name)
        out = '%s\n%s\n%s\n\n' \
              % (line, self.name, line)
        out = '%s%s\n\n' % (out, self.pprintcmd(self.cmd))
        if self.cmdout: out = '%s%s' % (out, self.pprintout(self.cmdout))
        if self.cmderr: out = '%s%s' % (out, self.pprintout(self.cmderr, 'cmderr'))
        return out

    log = property(getLog)

    def pprintcmd(self, cmd):
        cmd = cmd.replace(' -', '\n\t-')
        lidx = cmd.find('-')
        ridx = cmd.rfind(' ')
        cmd = 'Parameters\n----------\n\n::\n\n\t%s\n\t%s' \
              % (cmd[lidx:ridx], cmd[ridx+1:])
        return cmd

    def pprintout(self, out, label='cmdout'):
        line = '-' * len(label)
        out = out.replace('\n', '\n\t')
        out = '%s\n%s\n\n::\n\n\t%s\n' \
              % (label, line, out)
        return out
    

# HBuild
class HBuild(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HBuild'
        self.usage = """USAGE: HBuild [options] wordList latFile

 Option                                       Default

 -b      binary lattice output                ASCII
 -m s    load matrix bigram from s            off
 -n s    load back-off bigram from s          off
 -s s1 s2 s1/s2 are bigram start/end labels   !ENTER !EXIT
 -t s1 s2 bracket word-loop/pair with s1 s2   off
 -u s    set unknown symbol to s              !NULL
 -w s    load word-pair grammar from s        off
 -x s    load multi-level lattice from s      off
 -z      ignore ngrams with unknown symbol    off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-b', '-m', '-n', '-s', '-t',
                            '-u', '-w', '-x', '-z', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# HCompV
class HCompV(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HCompV'
        self.usage = """USAGE: HCompV [options] [hmmFile] trainFiles...
 Option                                       Default

 -c dir  Set output directiry for CMV         none
 -f f    Output vFloor as f * global var      none
 -k s    spkr pattern for CMV                 none
 -l s    Set segment label to s               none
 -m      Update means                         off
 -o fn   Store new hmm def in fn (name only)  outDir/srcfn
 -p s    path pattern for CMV                 none
 -q nmv  output type flags for CMV            m
 -v f    Set minimum variance to f            0.0
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-c', '-f', '-k', '-l', '-m',
                            '-o', '-p', '-q', '-v', '-A',
                            '-B', '-C', '-D', '-F', '-G',
                            '-H', '-I', '-L', '-M', '-S',
                            '-T', '-V', '-X']

# HCopy
class HCopy(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HCopy'
        self.usage = """USAGE: HCopy [options] src [ + src ...] tgt ...

 Option                                       Default

 -a i     Use level i labels                  1
 -e t     End copy at time t                  EOF
 -i mlf   Save labels to mlf s                null
 -l dir   Output target label files to dir    current
 -m t     Set margin of t around x/n segs     0
 -n i [j] Extract i'th [to j'th] label        off
 -s t     Start copy at time t                0
 -t n     Set trace line width to n           70
 -x s [n] Extract [n'th occ of] label  s      off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -O      Set target data format to fmt        as config
 -P      Set target label format to fmt       as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-e', '-i', '-l', '-m',
                            '-n', '-s', '-t', '-x', '-A',
                            '-C', '-D', '-F', '-G', '-I',
                            '-L', '-O', '-P', '-S', '-T',
                            '-V', '-X']

# HDMan
class HDMan(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HDMan'
        self.usage = """USAGE: HDMan [options] newDict srcDict1 srcDict2 ... 

 Option                                       Default

 -a s    chars in s start comment lines       #
 -b s    define word boundary symbol          #
 -e dir  look for edit scripts in dir
 -g f    global dictionary is in file f       global.ded
 -h i j  skip 1st i lines of j'th dic file    0
 -i      include output symbols               off
 -j      include pronunciation probabilities  off
 -l s    write log file in file s             no logging
 -m      merge prons from all sources         first_only
 -n f    output union of all phones to f      off
 -o      disable dictionary output            enabled
 -p f    load phone list stored in f
 -t      tag output words with source         off
 -w f    load word list stored in f
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -Q      Print command summary
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-a', '-b', '-e', '-g', '-h',
                            '-i', '-j', '-l', '-m', '-n',
                            '-o', '-p', '-t', '-w', '-A',
                            '-C', '-D', '-Q', '-S', '-T',
                            '-V']

# HERest
class HERest(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HERest'
        self.usage = """USAGE: HERest [options] hmmList dataFiles...

 Option                                       Default

 -a      Use an input linear transform        off
 -c f    Mixture pruning threshold            10.0
 -d s    dir to find hmm definitions          current
 -h s    set output speaker name pattern   *.%%%
         to s, optionally set input and parent patterns
 -l N    set max files per speaker            off
 -m N    set min examples needed per model    3
 -o s    extension for new hmm files          as src
 -p N    set parallel mode to N               off
 -r      Enable Single Pass Training...       
         ...using two parameterisations       off
 -s s    print statistics to file s           off
 -t f [i l] set pruning to f [inc limit]      inf
 -u tmvwap  update t)rans m)eans v)ars w)ghts tmvw
                a)daptation xform p)rior used     
                s)semi-tied xform                 
 -v f    set minimum variance to f            0.0
 -w f    set mix weight floor to f*MINMIX     0.0
 -x s    extension for hmm files              none
 -z s    Save all xforms to TMF file s        TMF
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -E s [s] set dir for parent xform to s       off
         and optional extension                 
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -J s [s] set dir for input xform to s        none
         and optional extension                 
 -K s [s] set dir for output xform to s       none
         and optional extension                 
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-c', '-d', '-h', 'to',
                            '-l', '-m', '-o', '-p', '-r',
                            '...using', '-s', '-t', '-u', 'a)daptation',
                            's)semi-tied', '-v', '-w', '-x', '-z',
                            '-A', '-B', '-C', '-D', '-E',
                            'and', '-F', '-G', '-H', '-I',
                            '-J', 'and', '-K', 'and', '-L',
                            '-M', '-S', '-T', '-V', '-X']

# HHEd
class HHEd(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HHEd'
        self.usage = """USAGE: HHEd [options] editF hmmList

 Option                                       Default

 -d s    dir to find hmm definitions          current
 -o s    extension for new hmm files          as source
 -w mmf  Save all HMMs to macro file mmf s    as source
 -x s    extension for hmm files              none
 -z      zap aliases in hmmList
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -H mmf  Load HMM macro file mmf
 -M dir  Dir to write HMM macro files         current
 -Q      Print command summary
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-d', '-o', '-w', '-x', '-z',
                            '-A', '-B', '-C', '-D', '-H',
                            '-M', '-Q', '-S', '-T', '-V']

# HInit
class HInit(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HInit'
        self.usage = """USAGE: HInit [options] hmmFile trainFiles...

 Option                                       Default

 -e f    Set convergence factor epsilon       1.0E-4
 -i N    Set max iterations to N              20
 -l s    Set segment label to s               none
 -m N    Set min segments needed              3
 -n      Update hmm (suppress uniform seg)    off
 -o fn   Store new hmm def in fn (name only)  outDir/srcfn
 -u mvwt Update m)eans v)ars w)ghts t)rans    mvwt
 -v f    Set minimum variance to f            1.0E-2
 -w f    set mix wt/disc prob floor to f      0.0
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-e', '-i', '-l', '-m', '-n',
                            '-o', '-u', '-v', '-w', '-A',
                            '-B', '-C', '-D', '-F', '-G',
                            '-H', '-I', '-L', '-M', '-S',
                            '-T', '-V', '-X']

# HLEd
class HLEd(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HLEd'
        self.usage = """USAGE: HLEd [options] edCmdFile labFiles...

 Option                                       Default

 -b      suppress boundary times              off
 -d s    load dictionary from s               off
 -i s    Output transcriptions to MLF s       off
 -g f    Set min gap for FG command to f      50000.0
 -l s    Dir to store output label file(s)    current
 -m      Strip to monophones on loading       off
 -n f    Output list of all new labs to f     off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -P      Set target label format to fmt       as config
 -Q      Print command summary
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-b', '-d', '-i', '-g', '-l',
                            '-m', '-n', '-A', '-C', '-D',
                            '-G', '-I', '-P', '-Q', '-S',
                            '-T', '-V', '-X']

# HList
class HList(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HList'
        self.usage = """USAGE: HList [options] file ...

 Option                                       Default

 -d      Coerce observation to VQ symbols     off
 -e N    End at sample N                      0
 -h      Print source header info             off
 -i N    Set items per line to N              10
 -n N    Set num streams to N                 1
 -o      Print observation structure          off
 -p      Playback audio                       off
 -r      Write raw output                     off
 -s N    Start at sample N                    0
 -t      Print target header info             off
 -z      Suppress printing data               on
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-d', '-e', '-h', '-i', '-n',
                            '-o', '-p', '-r', '-s', '-t',
                            '-z', '-A', '-C', '-D', '-F',
                            '-S', '-T', '-V']

# HLMCopy
class HLMCopy(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HLMCopy'
        self.usage = """USAGE: HLMCopy [options] inModel outModel

 Option                                       Default

 -c n c  set pruning for n-gram to c          0
 -d s    read source dict from s              none
 -f s    set output LM format to s            BIN
 -m      allow multiple identical prons       all
 -n n    save model as n-gram                 max
 -o      copy first dictionary pron to output all
 -u s    read new unigrams from s             none
 -v s    write pruned dictionary to s         none
 -w fn   prune model using word list in fn    off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-c', '-d', '-f', '-m', '-n',
                            '-o', '-u', '-v', '-w', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# HLRescore
class HLRescore(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HLRescore'
        self.usage = """USAGE: HLRescore [options] vocabFile Files...

 Option                                   Default

 -i s    Output transcriptions to MLF s       off
 -l s    dir to store label/lattice files     current
 -m s    merge nodes and arcs of lattice      off
 -n s    load n-gram LM and expand lattice    off
 -wn s   load word pair LM network            off
 -o s    output label formating NCSTWMX       none
 -t f [f] input lattice pruning threshold     off
 -u f [f] output lattice pruning threshold    off
 -p f    inter model trans penalty (log)      0.0
 -s f    grammar scale factor                 1.0
 -a f    acoustic scale factor                1.0
 -r f    pronunciation scale factor           1.0
 -d      get pronprobs from dict              off
 -c      calculate statistics                 off
 -f      find 1-best transcription            off
 -w      write output lattices                off
 -q s    output lattice format                tvaldmnr
 -y s    output label file extension          rec
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -P      Set target label format to fmt       as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-i', '-l', '-m', '-n', '-wn',
                            '-o', '-t', '-u', '-p', '-s',
                            '-a', '-r', '-d', '-c', '-f',
                            '-w', '-q', '-y', '-A', '-C',
                            '-D', '-G', '-I', '-L', '-P',
                            '-S', '-T', '-V', '-X']

# HLStats
class HLStats(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HLStats'
        self.usage = """USAGE: HLStats [options] hmmList labFile...

 Option                                       Default

 -b fn    output bigram to file fn            off
 -c N     count num logical occs upto N       none
 -d       compute duration statistics         off
 -f f     set matrix bigram floor prob f      0.0
 -h N     set hashsize: medium(1), large(2)   small(0)
 -l s     output covering list of models to s off
 -o       generate wsj style back-off files   matrix
 -p N     count num physical occs upto N      none
 -s s1 s2 select start s1 and end s2 labels   !ENTER !EXIT
 -t n     set threshold for including bigram  0
 -u f     set back off unigram floor prob f   1.0
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-b', '-c', '-d', '-f', '-h',
                            '-l', '-o', '-p', '-s', '-t',
                            '-u', '-A', '-C', '-D', '-G',
                            '-I', '-S', '-T', '-V', '-X']

# HMMIRest
class HMMIRest(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HMMIRest'
        self.usage = """USAGE: HMMIRest [options] hmmList dataFiles...

 Option                                   Default

 -d s    dir to find hmm definitions       current
 -D f    dictionary file.                  none   
 -g      MLE updates only.                   
 -l N    set max sentences (useful for debug) all
 -m N    set min examples needed per model   3
 -o s    extension for new hmm files        as src
 -p N    set parallel mode to N             off
 -q dir   Directory for numerator lats.        [needed. May use >1 -q option]
 -qp s   Subdirectory pattern string for numerator lats.    none
 -r dir   Directory for denominator lats.       [needed. May use >1 -r option]
 -rp s   Subdirectory pattern string for denominator lats.  none
 -s s    print statistics to file s         off
 -three  (in re-est phase) expect 3 accs, for MPE  off
 -twodatafiles  two data files for single-pass retrain  off
 -u tmvw update t)rans m)eans v)ars w)ghts...
 -umle tmvw do ML updates for specified parameters.
 -v f    set minimum variance to f          0.0
 -w f    set mix weight floor to f*MINMIX   0.0
 -x s    extension for hmm files            none
 -z   combine all accs into one acc, HDR0.acc.{1,2,3}  off 
 -Q      Lattice file extension              lat 
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab
 Note: doesn't work if means and variances are shared independently.

"""
        self.switch_keys = ['-d', '-D', '-g', '-l', '-m',
                            '-o', '-p', '-q', '-qp', '-r',
                            '-rp', '-s', '-three', '-twodatafiles', '-u',
                            '-umle', '-v', '-w', '-x', '-z',
                            '-Q', '-A', '-B', '-C', '-D',
                            '-F', '-H', '-I', '-M', '-S',
                            '-T', '-V', '-X', 'Note:']

# HParse
class HParse(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HParse'
        self.usage = """USAGE: HParse [options] netFile latFile

 Option                                       Default

 -b      output lattice in binary             ascii
 -c      set V1.x compatability mode          off
 -d s    output dictionary to file s          none
 -l      include LM log probs in lattice      off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-b', '-c', '-d', '-l', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# HQuant
class HQuant(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HQuant'
        self.usage = """USAGE: HQuant [options] vqFile trainFiles...

 Option                                       Default

 -d      Use diagonal cov Mahalanobis         Euclidean
 -f      Use full covariance Mahalanobis      Euclidean
 -g      Output global covar to codebook      off
 -l s    Set segment label to s               none
 -n S N  Set codebook size for stream S to N  N=256
 -s N    Set number of streams to N           1
 -t      Create tree-stuctured codebooks      linear
 -w S N  Set width of stream S to N           default
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-d', '-f', '-g', '-l', '-n',
                            '-s', '-t', '-w', '-A', '-C',
                            '-D', '-F', '-G', '-I', '-L',
                            '-S', '-T', '-V', '-X']

# HRest
class HRest(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HRest'
        self.usage = """USAGE: HRest [options] hmmFile trainFiles...

 Option                                       Default

 -e f    Set convergence factor epsilon       1.0E-4
 -i N    Set max iterations to N              20
 -l s    Set segment label to s               none
 -m N    Set min segments needed              3
 -t      Disable short segment rejection      on
 -u tmvw Update t)rans m)eans v)ars w)ghts    tmvw
 -v f    Set minimum variance to f            0.0
 -c f    Tied Mixture pruning threshold       10.0
 -w f    Set mix wt floor to f x MINMIX       0.0
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-e', '-i', '-l', '-m', '-t',
                            '-u', '-v', '-c', '-w', '-A',
                            '-B', '-C', '-D', '-F', '-G',
                            '-H', '-I', '-L', '-M', '-S',
                            '-T', '-V', '-X']

# HResults
class HResults(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HResults'
        self.usage = """USAGE: HResults [options] labelList recFiles...

 Option                                       Default

 -a s    Redefine string level label          SENT
 -b s    Redefine unitlevel label             WORD
 -c      Ignore case differences              off
 -d N    Find best of N levels                1
 -e s t  Label t is equivalent to s
 -f      Enable full results                  off
 -g fmt  Set test label format to fmt         HTK
 -h      Enable NIST style formatting         off
 -k s    Results per spkr using mask s        off
 -m N    Process only the first N rec files   all
 -n      Use NIST alignment procedure         off
 -p      Output phoneme statistics            off
 -s      Strip triphone contexts              off
 -t      Output time aligned transcriptions   off
 -u f    False alarm time units (hours)       1.0
 -w      Enable word spotting analysis        off
 -z s    Redefine null class name to s        ???
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-b', '-c', '-d', '-e',
                            '-f', '-g', '-h', '-k', '-m',
                            '-n', '-p', '-s', '-t', '-u',
                            '-w', '-z', '-A', '-C', '-D',
                            '-G', '-I', '-L', '-S', '-T',
                            '-V', '-X']

# HSGen
class HSGen(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HSGen'
        self.usage = """USAGE: HSGen [options] latticeFile dictFile

 Option                                       Default

 -l      Include line numbers                 off
 -n N    Number of sentences to generate      100
 -q      Suppress sentence output             off
 -s      Compute grammar stats                off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-l', '-n', '-q', '-s', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# HSLab
class HSLab(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HSLab'
        self.usage = """USAGE: HSLab [options] waveformFile

 Option                                       Default

 -a      auto-increment global label          off
 -i s    Output transcriptions to MLF s       off
 -n      Create new transcription             off
 -s str  Set special button label             command
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -P      Set target label format to fmt       as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-i', '-n', '-s', '-A',
                            '-C', '-D', '-F', '-G', '-I',
                            '-L', '-P', '-S', '-T', '-V',
                            '-X']

# HSmooth
class HSmooth(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HSmooth'
        self.usage = """USAGE: HSmooth [options] hmmList AccFiles...

 Option                                       Default

 -b f    set convergence epsilon              0.0001
 -c N    max num steps in binary chop         16
 -d s    dir to find hmm definitions          current
 -m N    set min examples needed per model    3
 -o s    extension for new hmm files          as src
 -s s    print statistics to file s           off
 -u tmvw Update t)rans m)eans v)ars w)ghts    tmvw
 -v f    Set minimum variance to f            0.0
 -w f    Set mix weight floor to f*MINMIX     0.0
 -x s    extension for hmm files              none
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -H mmf  Load HMM macro file mmf
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-b', '-c', '-d', '-m', '-o',
                            '-s', '-u', '-v', '-w', '-x',
                            '-A', '-B', '-C', '-D', '-H',
                            '-M', '-S', '-T', '-V']

# HVite
class HVite(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HVite'
        self.usage = """USAGE: HVite [options] VocabFile HMMList DataFiles...

 Option                                       Default

 -a      align from label files               off
 -b s    def s as utterance boundary word     none
 -c f    tied mixture pruning threshold       10.0
 -d s    dir to find hmm definitions          current
 -e      save direct audio rec output         off
 -f      output full state alignment          off
 -g      enable audio replay                  off
 -h s    set speaker name pattern             *.mfc
 -i s    Output transcriptions to MLF s       off
 -j i    Online MLLR adaptation               off
         Perform update every i utterances      
 -k      use an input transform               off
 -l s    dir to store label/lattice files     current
 -m      output model alignment               off
 -n i [N] N-best recognition (using i tokens) off
 -o s    output label formating NCSTWMX       none
 -p f    inter model trans penalty (log)      0.0
 -q s    output lattice formating ABtvaldmn   tvaldmn
 -r f    pronunciation prob scale factor      1.0
 -s f    grammar scale factor                 1.0
 -t f [f f] set pruning threshold             0.0
 -u i    set pruning max active               0
 -v f    set word end pruning threshold       0.0
 -w [s]  recognise from network               off
 -x s    extension for hmm files              none
 -y s    output label file extension          rec
 -z s    generate lattices with extension s   off
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -E s [s] set dir for parent xform to s       off
         and optional extension                 
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -J s [s] set dir for input xform to s        none
         and optional extension                 
 -K s [s] set dir for output xform to s       none
         and optional extension                 
 -L dir  Set input label (or net) dir         current
 -P      Set target label format to fmt       as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-b', '-c', '-d', '-e',
                            '-f', '-g', '-h', '-i', '-j',
                            'Perform', '-k', '-l', '-m', '-n',
                            '-o', '-p', '-q', '-r', '-s',
                            '-t', '-u', '-v', '-w', '-x',
                            '-y', '-z', '-A', '-B', '-C',
                            '-D', '-E', 'and', '-F', '-G',
                            '-H', '-I', '-J', 'and', '-K',
                            'and', '-L', '-P', '-S', '-T',
                            '-V', '-X']

# LAdapt
class LAdapt(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LAdapt'
        self.usage = """USAGE: LAdapt [options] langModel txtfile ....

 Option                                       Default

 -a n    allow n new words in input text      100000
 -b n    set ngram buffer size                2000000
 -c n c  set pruning for n-gram to c          1
 -d s    set root n-gram data file name       gram
 -f s    set output LM format to s            BIN
 -g      use existing n-gram files            off
 -i f s  interpolate with model s, weight f   off
 -j n c  set weighted discount pruning to c   off
 -n n    set n-gram size                      3
 -s s    store s in gram header source flds   none
 -t      use Turing-Good discounting          off
 -w fn   load word list from fn               none
 -x      save model with counts               off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-a', '-b', '-c', '-d', '-f',
                            '-g', '-i', '-j', '-n', '-s',
                            '-t', '-w', '-x', '-A', '-C',
                            '-D', '-S', '-T', '-V']

# LBuild
class LBuild(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LBuild'
        self.usage = """USAGE: LBuild [options] wordMap langModel gramfile ....

 Option                                       Default

 -c n c  set cutoff for n-gram to c           1
 -d n c  set weighted discount pruning to c   off
 -f s    set output LM format to s            BIN
 -k n    set discounting range to [1..n]      7
 -l fn   build from existing LM from fn       off
 -n n    set model order                      max
 -t fn   load FoF table from fn               off
 -u n    set unigram floor count to n         1
 -x      save model with counts               off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-c', '-d', '-f', '-k', '-l',
                            '-n', '-t', '-u', '-x', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# LFoF
class LFoF(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LFoF'
        self.usage = """USAGE: LFoF [options] wordMap fofFile <[mult] file ....>

 Option                                       Default

 -f n    set FoF table size to n              128
 -n n    set N-gram                           3
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-f', '-n', '-A', '-C', '-D',
                            '-S', '-T', '-V']

# LGCopy
class LGCopy(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LGCopy'
        self.usage = """USAGE: LGCopy [options] map <[mult] file ....>

 Option                                       Default

 -a n    allow up to 'n' new classes          100
 -b n    set ngram buffer size to 'n'         2000000
 -d s    database directory 's'               current
 -i n    set output gram file start index     0
 -m fn   save new word map to 'fn'            off
 -n n    create n-grams of size 'n'           max
 -o      output class mappings only           off
 -r s    set root data base file name 's'     data
 -w fn   load class map from 'fn'             none
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-a', '-b', '-d', '-i', '-m',
                            '-n', '-o', '-r', '-w', '-A',
                            '-C', '-D', '-S', '-T', '-V']

# LGList
class LGList(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LGList'
        self.usage = """USAGE: LGList [options] mapfile gramfile ...

 Option                                       Default

 -i n    add id n to filter list              none
 -f w    add word w to filter list            none
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-i', '-f', '-A', '-C', '-D',
                            '-S', '-T', '-V']

# LGPrep
class LGPrep(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LGPrep'
        self.usage = """USAGE: LGPrep [options] wmap [txtfile] ....

 Option                                       Default

 -a n    allow n new words in input text      100000
 -b n    set gram buffer size                 2000000
 -c      force counts into wordmap on update  same as input
 -d s    output gram file directory           current directory
 -e n    edited gram buffer size              100000
 -f s    fix text source using rules in s     off
 -h      disable HTK escaping on output       off
 -i n    set output gram file start index     0
 -n n    set n-gram size                      3
 -q      tag sentence start words with '_'    off
 -r s    set root gram filename               gram
 -s s    store s in gram header source fields none
 -w s    write output map to s                wmap [or -d dir/wmap]
 -z      suppress gram file generation        off
 -Q      print rule summary help              off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-a', '-b', '-c', '-d', '-e',
                            '-f', '-h', '-i', '-n', '-q',
                            '-r', '-s', '-w', '-z', '-Q',
                            '-A', '-C', '-D', '-S', '-T',
                            '-V']

# LLink
class LLink(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LLink'
        self.usage = """USAGE: LLink [options] word-classLMfile class-classLMfile outLMfile

 Option                                       Default

 -c      Force model to say 'counts'          as per w|c file
 -p      Force model to say 'probabilities'   as per w|c file
 -s      Merge into one file, not three       off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-c', '-p', '-s', '-A', '-C',
                            '-D', '-S', '-T', '-V']

# LMerge
class LMerge(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LMerge'
        self.usage = """USAGE: LMerge [options] wordList inModel outModel

 Option                                       Default

 -f s    set output LM format to s            BIN
 -i f s  interpolate with model s, weight f   off
 -n n    produce n-gram model                 max
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-f', '-i', '-n', '-A', '-C',
                            '-D', '-G', '-I', '-S', '-T',
                            '-V']

# LNewMap
class LNewMap(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LNewMap'
        self.usage = """USAGE: LNewMap [options] name mapfn

 Option                                       Default

 -e esc  Set escape mode to 'esc'             RAW
 -f fld  Add field 'fld' to Fields header     ID
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-e', '-f', '-A', '-C', '-D',
                            '-S', '-T', '-V']

# LNorm
class LNorm(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LNorm'
        self.usage = """USAGE: LNorm [options] inModel outModel

 Option                                       Default

 -c n c  set cutoff for n-gram to c           0
 -d n c  set weighted discount pruning to c   off
 -f s    set output LM format to s            BIN
 -n n    save model as n-gram                 max
 -w fn   prune model using word list in fn    off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-c', '-d', '-f', '-n', '-w',
                            '-A', '-C', '-D', '-G', '-I',
                            '-S', '-T', '-V']

# LPlex
class LPlex(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LPlex'
        self.usage = """USAGE: LPlex [options] langmodel labelFiles...

 Option                                       Default

 -c n c  set pruning for n-gram to c          off
 -d n c  set weighted discount pruning to c   off
 -e s t  Label t is equivalent to s           off
 -i f s  interpolate with model s, weight f   off
 -n N    calculate N-gram perplexity          max in LM
 -o      print OOV word statistics            off
 -s fn   print prob stream to file fn         off
 -t      text stream mode                     off
 -u      use OOV words in context             off
 -w fn   use word list from fn                off
 -z s    Redefine null class name to s        ???
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-c', '-d', '-e', '-i', '-n',
                            '-o', '-s', '-t', '-u', '-w',
                            '-z', '-A', '-C', '-D', '-G',
                            '-I', '-S', '-T', '-V']

# LSubset
class LSubset(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'LSubset'
        self.usage = """USAGE: LSubset [options] inMapFile classMap outMapFile

 Option                                       Default

 -a n    allow up to n new classes in map     1000
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off

"""
        self.switch_keys = ['-a', '-A', '-C', '-D', '-S',
                            '-T', '-V']


def main():
    c = LSubset()
    c.run()
    print c.getLog()

if __name__ == '__main__':
    main()
 
